import json
from celery import Celery
from django.db import models
from redbeat import RedBeatSchedulerEntry as Entry
from celery.schedules import schedule
from celery import current_app
# Create your models here.
class PeriodicTasksEntry(models.Model):

    key = models.CharField(max_length=255, null=True, blank=True)
    name = models.CharField(max_length=255, null=True, blank=True)
    task = models.CharField(max_length=255, null=True, blank=True)
    enabled = models.BooleanField(default=True)
    schedule = models.IntegerField(default=10, help_text="The schedule in seconds.")
    args = models.TextField(blank=True, null=True)

    def __str__(self):
        return self.name

    def save(self, force_insert=False, force_update=False, *args, **kwargs):
        self.key = self.add_periodic_task(args_list=self.get_args())
        instance = super(PeriodicTasksEntry, self).save(force_insert, force_update, *args, **kwargs)
        return instance

    def get_args(self):
        return json.loads(str(self.args))

    def get_entry(self):
        try:
            entry = Entry.from_key(self.key, app=current_app)
            return entry
        except Exception as e:
            return None

    def add_periodic_task(self, args_list, *args, **kwargs):
        try:
            # TASK ALREADY EXISTS IN REDIS
            entry = Entry.from_key(key=self.key)
            return entry.key
        except Exception as e:
            # TASK DOESNT EXIST IN REDIS
            entry = Entry(name=self.name, task=self.task, args=args_list, schedule=self.schedule, app=current_app)
            entry.save()
            self.key = entry.key
            return self.key

    def delete_periodic_task(self):
        entry = self.get_entry()
        if entry:
            entry.delete()
    
    def update_schedule(self, *args, **kwargs):
        entry = Entry.from_key(self.key, app=current_app)
        entry.schedule = schedule(run_every=self.schedule)
        entry.save()

    class Meta:
        verbose_name = 'Periodic Tasks Entry'
        verbose_name_plural = 'Periodic Tasks Entries'



# Generated by Django 4.1.2 on 2022-10-23 15:22

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PeriodicTasksEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(blank=True, max_length=255, null=True)),
                ('name', models.CharField(blank=True, max_length=255, null=True)),
                ('task', models.CharField(blank=True, max_length=255, null=True)),
                ('enabled', models.BooleanField(default=True)),
                ('schedule', models.IntegerField(default=10, help_text='The schedule in seconds.')),
                ('args', models.TextField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Periodic Tasks Entry',
                'verbose_name_plural': 'Periodic Tasks Entries',
            },
        ),
    ]



from django.dispatch import receiver
from django_redbeat.models import PeriodicTasksEntry
from django.db.models.signals import post_delete, post_save, pre_delete, pre_save
from redbeat import RedBeatSchedulerEntry as Entry


@receiver(post_save, sender=PeriodicTasksEntry)
def handle_periodic_task_admin(sender, instance, created, **kwargs):
    entry = instance.get_entry()
    if instance.enabled == False:
        # DISABLED THE TASK
        instance.delete_periodic_task()
    elif instance.enabled == True and entry is None:
        # ENABLED THE TASK
        instance.add_periodic_task()
        #entry.reschedule(instance.schedule)


"""
Signal to delete celery-redbeat  periodic tasks from django admin
"""
@receiver(pre_delete, sender=PeriodicTasksEntry)
def delete_periodic_task_entry_admin(sender, instance, using, **kwargs):
    entry = None
    try:
        entry = instance.get_entry()
    except Exception:
        pass
    if entry is not None:
        instance.delete_periodic_task()


@receiver(pre_save, sender=PeriodicTasksEntry)
def update_schedule(sender, instance, **kwargs):
    try:
        obj = sender.objects.get(pk=instance.pk)
    except sender.DoesNotExist:
        print('Perioidc Task is new!')
        # Object is new, so field hasn't technically changed
    else:
        if not obj.schedule == instance.schedule : # TODO TEST THIS PROPERLY !!
              # Schedule field has changed
              instance.update_task_schedule()